#------------------------------------------------------------------------------
# TUAM Admin console deployment/undeployment script
#
# IBM Tivoli Usage and Accounting Manager
# AUC
# (C) Copyright IBM Corp. 2007
#
# Installing the console:
# wsadmin.sh -f deployTUAMConsole.py install
#
# Uninstalling the console:
# wsadmin.sh -f deployTUAMConsole.py remove
#------------------------------------------------------------------------------
import sys
import os.path


#------------------------------------------------------------------------------
# Get the directory, as a string, where WAS is installed.
#------------------------------------------------------------------------------
def getWASHome(cell, node):
	varMap = AdminConfig.getid("/Cell:" + cell + "/Node:" + node + "/VariableMap:/")
	entries = AdminConfig.list("VariableSubstitutionEntry", varMap)
	eList = entries.splitlines()
	for entry in eList:
		name =  AdminConfig.showAttribute(entry, "symbolicName")
		print name
		if name == "WAS_INSTALL_ROOT":
			value = AdminConfig.showAttribute(entry, "value")
			return value
	#failover
	return java.lang.System.getenv('WAS_HOME')


#------------------------------------------------------------------------------
# Get the directory, as a string, of the isclite.ear application.
#
# The isc.ear is located in <WAS_HOME>/profiles/TIPProfile/installedApps/TIPCell/isc.ear
#------------------------------------------------------------------------------
def getISCFile(cell, node):
	fileSep = getFileSep(node)
	return fileSep.join([getWASHome(cell, node), "profiles", "TIPProfile", "installedApps", cell, "isc.ear", "aucConsole.war"])
	
#------------------------------------------------------------------------------
# Get the file separator character
#
# This gets the file separator for the node in which we plan to install the
# console.  Therefore we can't just use the value that python on Java gives
# us.  Instead, check the platform of the node, and use "\" for windows and
# "/" for everything else.
#------------------------------------------------------------------------------
def getFileSep(node):
	os = AdminTask.getNodePlatformOS("-nodeName " + node)
	if os == 'windows':
		return '\\'
	else:
		return '/'

#------------------------------------------------------------------------------
# setCellVar
#------------------------------------------------------------------------------
def setCellVar(cell):
	try:
		varMap = AdminConfig.getid("/Cell:" + cell + "/VariableMap:/")
		prop = [[['symbolicName', 'WAS_CELL_NAME'], ['value', cell]]]
		AdminConfig.modify(varMap, [['entries', prop]])
		return 1
	except:
		print "Error during setCellVar(" + cell + "):", sys.exc_info()
		return 0



#------------------------------------------------------------------------------
# deployTUAMAdminConsole
# Deploy the aucConsole.war using the AdminApp update command, and then map it
# to the admin_host virtual host.
#------------------------------------------------------------------------------
def deployTUAMAdminConsole (cell, node, server, type):
	warFile  = getISCFile (cell, node)
	try:
		print "Deploying aucConsole.war"
		AdminApp.update('isc' , 'modulefile', ['-operation', 'add', '-contents', warFile, '-contenturi', 'aucConsole.war', '-usedefaultbindings', '-contextroot' , '/tuamConsole', '-MapWebModToVH', [['.*', '.*', 'default_host']]])

	except:
		error = str(sys.exc_info()[1])
		if error.count("7279E") > 0: # catch WASX7279E (app with given name already exists)
			print "TUAM Administration console is already installed."
		else:
			print "Exception occurred during deployTuamConsole(" + cell +", " + node + ", " + server + "):", error
		return 0
	return 1

def addJarToWar (jar, jardir):
	try:
		print "Adding " + jar
		AdminApp.update('isc', 'file', ['-operation', 'add', '-contents', jardir + '/' + jar, '-contenturi', 'aucConsole.war/WEB-INF/lib/' + jar])

	except:
		error = str(sys.exc_info()[1])
		if error.count("7279E") > 0: # catch WASX7279E (app with given name already exists)
			print "TUAM Administration console is already installed."
		else:
			print "Exception occurred during deployTuamConsole(" + cell +", " + node + ", " + server + "):", error
		return 0
	return 1

#------------------------------------------------------------------------------
# Get a tuple containing the cell, node, server name, and type
#------------------------------------------------------------------------------
def getCellNodeServer():
	servers = AdminConfig.list("Server").splitlines()
	for serverId in servers:
		serverName = serverId.split("(")[0]
		server = serverId.split("(")[1]  #remove name( from id
		server = server.split("/")
		cell = server[1]
		node = server[3]
		cellId = AdminConfig.getid("/Cell:" + cell + "/")
		cellType = AdminConfig.showAttribute(cellId, "cellType")
		if cellType == "DISTRIBUTED":
			if AdminConfig.showAttribute(serverId, "serverType") == "DEPLOYMENT_MANAGER":
				return (cell, node, serverName, "DEPLOYMENT_MANAGER")
		elif cellType == "STANDALONE":
			if AdminConfig.showAttribute(serverId, "serverType") == "APPLICATION_SERVER":
				return (cell, node, serverName, "APPLICATION_SERVER")
	return None


#------------------------------------------------------------------------------
# Print script usage
#------------------------------------------------------------------------------
def printUsage():
	print "Usage: wsadmin deployTUAMConsole.py install"
	print "  or:  wsadmin deployTUAMConsole.py remove"
	print ""

#------------------------------------------------------------------------------
# Install the admin console
#------------------------------------------------------------------------------
def doInstall(jardir):
	topology = getCellNodeServer()
	if topology == None:
		sys.stderr.write("Could not find suitable server\n")
		if failOnError == "true":
			sys.exit(105)
	else:
		cell = topology[0]
		node = topology[1]
		server = topology[2]
		type = topology[3]
		
		retVal = deployTUAMAdminConsole(cell, node, server, type)
		if retVal == 1:
			print "Checking for jars"
			for jar in ['aucBCP.jar', 'aucECP.jar', 'aucNFC.jar']:
				if os.path.exists(os.path.join(jardir, jar)):
					retVal = addJarToWar(jar, jardir)
					if retVal != 1:
						break
		if retVal == 1:
			AdminConfig.save()
		else:
			print "Skipping Config Save"
			if failOnError == "true":
				sys.exit(109)


#------------------------------------------------------------------------------
# Uninstall the admin console
#------------------------------------------------------------------------------
def doRemove():

	AdminApp.update('isc' , 'modulefile', ['-operation', 'delete', '-contenturi', 'aucConsole.war'])
	AdminConfig.save()


#------------------------------------------------------------------------------
# Main entry point
#------------------------------------------------------------------------------

failOnError = "false"
jardir = None

if len(sys.argv) < 1 or len(sys.argv) > 2:
	sys.stderr.write("Invalid number of arguments\n")
	printUsage()
	sys.exit(101)
else:
	print sys.argv
	if len(sys.argv) == 2:
		if sys.argv[1] == "-failonerror":
			failOnError = "true"
			print "failonerror is enabled"
		else:
			jardir = sys.argv[1]

	mode = sys.argv[0]
	if mode == "install":
		print "Installing TUAM Administration Console..."
		doInstall(jardir)
	elif mode == "remove":
		print "Removing TUAM Administration Console..."
		doRemove()
	else:
		sys.stderr.write("Invalid command:  " + mode + "\n")
		printUsage()
		if failOnError == "true":
			sys.exit(103)



